/******************************************************************************
 * w_scan_cpp - a dtv channel scanner based on VDR (www.tvdr.de) and it's
 * Plugins.
 *
 * See the README file for copyright information and how to reach the author.
 *****************************************************************************/
#include "Helpers.h"
#include "CmdOpts.h"
#include "Satellites.h"
#include "Countries.h"
#include "Library.h"
#include "Lnb.h"
#include "DiSEqC.h"
#include "ScanControl.h"
#include "OutputFormats.h"
#include "Version.h"

int main(int argc, char* argv[]) {
  Message("/*******************************************************************************");
  Message(" * w_scan_cpp Version " + version);
  Message(" ******************************************************************************/\n");

  if (not(ParseArguments(argc, argv))) {
     ErrorMessage("Error reading command line arguments. Exit!");
     return -1;
     }

  if (WirbelscanSetup.HelpText) {
     UnloadLibraries();
     return 0;
     }

  InitCharTables();

  for(auto l:libs) {
     l->Plugin()->Initialize();
     l->Plugin()->Start();
     }

  Message("/*******************************************************************************");
  Message(" * devices");
  Message(" ******************************************************************************/");
  if (satip == nullptr) {
     if (WirbelscanSetup.adapter.empty()) {
        cDvbDevice::Initialize();
        }
     else {
        if (WirbelscanSetup.adapter.find("/dev/dvb/adapter") == 0) {
           std::string s(WirbelscanSetup.adapter);
           s.erase(0, std::string("/dev/dvb/adapter").size());
           int a = std::stoi(s.substr(0, s.find("frontend")-1));
           s.erase(0, s.find("frontend") + std::string("frontend").size());
           int f = std::stoi(s);
           new cDvbDevice(a, f);
           }
        else if (WirbelscanSetup.adapter.find_first_of("0123456789") == 0) {
           int a = std::stoi(WirbelscanSetup.adapter);
           new cDvbDevice(a, 0);
           }
        }
     }
  else {
     Message("skip DVB devices - using satip plugin.");
     Sleep(3000);
     }

  while (!cDevice::WaitForAllDevicesReady(3));

  for(int i = 0; i < cDevice::NumDevices(); i++) {
     cDevice* d = cDevice::GetDevice(i);
     cDvbDevice* dvb = dynamic_cast<cDvbDevice*>(d);

     std::string name = "Device" +  std::to_string(i);
     if (d->NumProvidedSystems() < 1) {
        name += ": Device or resource busy";
        }        
     else if (*d->DeviceName() != nullptr) {
        if (dvb) {
           name += " dvb:";
           name += "a" + std::to_string(dvb->Adapter());
           name += "f" + std::to_string(dvb->Frontend());
           name += ": ";
           }
        else
           name += " plg: ";
        name += *d->DeviceName();
        }
     Message(name);
     }

  Message("/*******************************************************************************");
  Message(" * Scan Type");
  Message(" ******************************************************************************/");
  switch(WirbelscanSetup.DVB_Type) {
     case 5 /* A */: {
        Message("  ATSC");
        Message("  country = " + CountryNames[WirbelscanSetup.CountryIndex]);
        break;
        }
     case 1 /* C */: {
        Message("  DVB-C");
        Message("  country = " + CountryNames[WirbelscanSetup.CountryIndex]);
        break;
        }
     case 2 /* S */: {
        Message("  DVB-S/S2");
        Message("  satellite = " + SatNames[WirbelscanSetup.SatIndex]);
        PrintLnb();
        if (Setup.DiSEqC)
           PrintDiseqc();
        break;
        }
     default/* T */: {
        Message("  DVB-T/T2");
        Message("  country = " + CountryNames[WirbelscanSetup.CountryIndex]);
        break;
        }
     }

  bool Continue = StartScan();

  while(Continue) {
     Sleep(500);
     Continue = Scanning();
     }

  std::vector<TChannel> data;
  if (Import(data)) {
     if      (OutputFormat == "VDR")     PrintVDR(data);
     else if (OutputFormat == "INI")     PrintIni(data);
     else if (OutputFormat == "VLC")     PrintVLC(data);
     else if (OutputFormat == "XINE")    PrintXine(data);
     else if (OutputFormat == "MPLAYER") PrintXine(data, true);
     else if (OutputFormat == "INITIAL") PrintInitial(data);
     else if (OutputFormat == "XML")     PrintXML(data);
     else if (OutputFormat == "DAT")     PrintDat(data, WirbelscanSetup.OutputFile);
     else Message("no output format defined - try again!");
     }
  else
     ErrorMessage("Import failed.");

  for(auto l:libs)
     l->Plugin()->Stop();

  UnloadLibraries();
}
